/*
 *  Copyright (c) 2010-2013, Rutgers, The State University of New Jersey
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the organization(s) stated above nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file   mfmain.cpp
 * @author wontoniii (bronzino@winlab.rutgers.edu), czhang
 * @date   July, 2011
 * @brief  Temp.
 *
 * Temp.
 */

#include "mfmain.h"

void threadExitHandler(int sig) {
	//printf("threadExitHandler(): thead id = %d\n", pthread_self());
	//pthread_exit(0);
}

MF_Main::MF_Main() : mNetManager(&mEvQueue), mRouter(&mNetManager, &mSockTable, &mEvQueue)  {
	cont = true;
	this->oml_mps = NULL;
}

#ifdef __OML__

MF_Main::MF_Main( MF_OML* oml_mps) : mNetManager(&mEvQueue), mRouter(&mNetManager, &mSockTable, &mEvQueue, oml_mps){
	this->oml_mps = oml_mps;
	cont = true;
}
#endif

MF_Main::~MF_Main() {

}

void MF_Main::start() {
	/*
	 * Start network manager thread
	 */
	mNetManager.start();
	if(mEvQueue.startListener(&ms)){
		cont = false;
	}
	MF_StackEvent *temp;
	while (cont) {
		temp = mEvQueue.getNext();
		handleMessage(temp);
	}
}

void MF_Main::start(MF_Settings *setts) {
	/*
	 * Start network manager thread
	 */
	MF_Log::mf_log(MF_DEBUG, "MF_Main::start(): starting stack");
	mSettings = setts;
	mNetManager.setFromSettings(mSettings, oml_mps);
	mRouter.setPolicy(mSettings->getPolicy());
	mRouter.setDefaultGUID(mSettings->getDefaultGUID());
	mNetManager.start();
	if(mEvQueue.startListener(&ms)){
		cont = false;
	}
	MF_StackEvent *temp;
	while (cont) {
		temp = mEvQueue.getNext();
		if(temp){
			handleMessage(temp);
		}
	}
}

void MF_Main::handleMessage(MF_StackEvent *event) {
	switch (event->m_type()) {
	//TODO Order cases by probability to increase performance...
	case EV_OPEN: //from API
		handleOpen((MF_APIOpen *)event);
		break;
	case EV_SEND: //from API
		handleSend((MF_APISend *)event);
		break;
	case EV_RECV: //from API
		handleRecv((MF_APIRecv *)event);
		break;
	case EV_GET: //from API
		handleGet((MF_APIGet *)event);
		break;
	case EV_DO_GET: //from API
		handleDoGet((MF_APIDoGet *)event);
		break;
	case EV_GET_RESPONSE: //from API
		handleGetResponse((MF_APIGetResponse *)event);
		break;
	case EV_ATTACH: //from API
		handleAttach((MF_APIAttach *)event);
		break;
	case EV_DETACH: //from API
		handleDetach((MF_APIDetach *)event);
		break;
	case EV_CLOSE: //from API
		handleClose((MF_APIClose *)event);
		break;
	case EV_CSYN_ACK: //from DeviceManager, (for send)
		handleCsynAck((MF_EvCysnAck *)event);
		break;
	case EV_CSYN: //from DeviceManager, (for send)
		handleCsyn((MF_EvCysn *)event);
		break;
	case EV_CSYN_TIMEOUT: //from Timer thread
		handleCsynTimeout((MF_EvCysnTOut *)event);
		break;
	case EV_IF_STATE_UPDATE: //from NetworkManager thread
		handleIfStateUpdate((MF_EvIfStateUpdate *)event);
		break;
	case EV_LP:
		handleLinkProbe((MF_EvLinkProb *)event);
		break;
	case EV_DOWN_API:
		handleDownAPI((MF_EvDownAPI *)event);
		break;
	case EV_DOWN_TRANSP:
		handleDownTransp((MF_EvDownTransp *)event);
		break;
	case EV_DOWN_ROUT:
		handleDownRout((MF_EvDownRout *)event);
		break;
	case EV_UP_IF:
		handleUpIf((MF_EvUpIf *)event);
		break;
	case EV_UP_ROUT:
		handleUpRout((MF_EvUpRout *)event);
		break;
	case EV_UP_TRANSP:
		handleUpTransp((MF_EvUpTransp *)event);
		break;
	case EV_CHK_SENT:
		handleChkSent((MF_EvChkComplete *)event);
		break;
	default:
		MF_Log::mf_log(MF_WARN, "MF_Main::handleMessage(): wrong message type...");
	}
	delete event;
}

/********************************
 Operations to handle the communication with the API
********************************/


/*
 * Handle open request
 */
void MF_Main::handleOpen(MF_APIOpen *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleOpen");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleOpen start of event");
	MF_SocketManager *man = mSockTable.getById(event->getUID());
	if(man!=NULL && man->isOpen()){
		MF_Log::mf_log(MF_INFO, "MF_Main:handleOpen An app with UID %lu already exists\n",event->getUID());
		man->openReplyError();
	}
	else{
		//TODO: Add resource management to check if there are enough resources
		MF_SocketManager * mSM = new MF_SocketManager(&mSockTable, &mEvQueue, &ms, event->getUID(), mSettings->getDefaultBufferSize());
		if(mSM->init() && mSM->openRequest(event->getProfile(), event->getOpts(), event->getSrcGUID()) == 0){
			mSM->openReplySuccess();
			mSM->connectToClient();
			//TODO find a cleaner solution than this
			if(event->getSrcGUID()==0){
				mSockTable.addGUID(event->getUID(), mRouter.getDefaultGUID());
			}
			else{
				mSockTable.addGUID(event->getUID(), event->getSrcGUID());
				mRouter.addGUID(event->getSrcGUID());
			}
		}
		else{
			mSM->openReplyError();
		}
	}
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleOpen end of event");
}


/*
 * Handle a receive request from the API level
 */
void MF_Main::handleRecv(MF_APIRecv *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleRecv");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleRecv start of event");
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(mSM == NULL){
		MF_Log::mf_log(MF_DEBUG, "MF_Main:handleRecv no socket manager for this UID");
		struct MsgRecvReply rcR;
		rcR.type = MSG_TYPE_RECV_REPLY;
		rcR.retValue = UID_NEXISTS;
		rcR.reqID = event->getReqID();
		ms.send((char*)&rcR, sizeof(struct MsgSendReply), event->getUID());
	}
	else{
		mSM->recvRequest(event);
	}
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleRecv start of event");
}


/*
 * Handles a send request from the API level
 */
void MF_Main::handleSend(MF_APISend *event) {
	MF_Log::mf_log(MF_TIME, "MF_Main:handleSend SEND_TIME request number [%lu] INIT",event->getReqID());
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(!mSM){
		struct MsgSendReply rsS;
		rsS.type = MSG_TYPE_SEND_REPLY;
		rsS.retValue = UID_NEXISTS;
		rsS.reqID = event->getReqID();
		ms.send((char*)&rsS, sizeof(struct MsgSendReply), event->getUID());
	}
	else{
		mSM->sendRequest(event);
	}
	MF_Log::mf_log(MF_TIME, "MF_Main:handleSend SEND_TIME request number [%lu] END",event->getReqID());
}

/*
 * Handles a get request from the API level
 */
void MF_Main::handleGet(MF_APIGet *event) {
	MF_Log::mf_log(MF_TIME, "MF_Main:handleGet GET_TIME request number [%lu] INIT",event->getReqID());
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(!mSM){
		struct MsgGetReply rsS;
		rsS.type = MSG_TYPE_GET_SEND_REPLY;
		rsS.retValue = UID_NEXISTS;
		rsS.reqID = event->getReqID();
		ms.send((char*)&rsS, sizeof(struct MsgGetReply), event->getUID());
	}
	else{
		mSM->getRequest(event);
	}
	MF_Log::mf_log(MF_TIME, "MF_Main:handleGet GET_TIME request number [%lu] END",event->getReqID());
}

/*
 * Handle a do get request from the API level
 */
void MF_Main::handleDoGet(MF_APIDoGet *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleDoGet");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleRecv start of event");
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(mSM == NULL){
		MF_Log::mf_log(MF_DEBUG, "MF_Main:handleDoGet no socket manager for this UID");
		struct MsgDoGetReply rcR;
		rcR.type = MSG_TYPE_DO_GET_REPLY;
		rcR.retValue = UID_NEXISTS;
		rcR.reqID = event->getReqID();
		ms.send((char*)&rcR, sizeof(struct MsgDoGetReply), event->getUID());
	}
	else{
		mSM->doGetRequest(event);
	}
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleRecv start of event");
}

/*
 * Handles a get response from the API level
 */
void MF_Main::handleGetResponse(MF_APIGetResponse *event) {
	MF_Log::mf_log(MF_TIME, "MF_Main:handleGetResponse GET_RESPONSE_TIME request number [%lu] INIT",event->getReqID());
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(!mSM){
		struct MsgGetResponseReply rsS;
		rsS.type = MSG_TYPE_GET_RESPONSE_REPLY;
		rsS.retValue = UID_NEXISTS;
		rsS.reqID = event->getReqID();
		ms.send((char*)&rsS, sizeof(struct MsgGetResponseReply), event->getUID());
	}
	else{
		mSM->getResponseRequest(event);
	}
	MF_Log::mf_log(MF_TIME, "MF_Main:handleGetResponse GET_RESPONSE_TIME request number [%lu] END",event->getReqID());
}

/*
 * Handle attach requests from the API
 */
void MF_Main::handleAttach(MF_APIAttach *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main: handleAttach");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleAttach start of event");
	struct MsgAttachReply mar;
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(!mSM){
		mar.type = MSG_TYPE_ATTACH_REPLY;
		mar.UID = event->getUID();
		mar.reqID = event->getReqID();
		mar.retValue = UID_NEXISTS;
		ms.send((char*)&mar, sizeof(struct MsgAttachReply), event->getUID());
	}
	else{
		for(int i=0; i<(int)event->getNGUID(); i++){
			//TODO check that only in one place it should be added to the socket manager
			//mSM->addGUID(event->getGUID(i));
			mSockTable.addGUID(event->getUID(), event->getGUID(i));
			mRouter.addGUID(event->getGUID(i));
		}
		mar.type = MSG_TYPE_ATTACH_REPLY;
		mar.UID = event->getUID();
		mar.reqID = event->getReqID();
		mar.retValue = 0;
		ms.send((char*)&mar, sizeof(struct MsgAttachReply), event->getUID());
	}
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleAttach end of event");
}

/*
 * Handle detach requests from the API
 */
void MF_Main::handleDetach(MF_APIDetach *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleDetach");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleDetach start of event");
	struct MsgDetachReply mdr;
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	if(!mSM){
		mdr.type = MSG_TYPE_DETACH_REPLY;
		mdr.UID = event->getUID();
		mdr.reqID = event->getReqID();
		mdr.retValue = UID_NEXISTS;
		ms.send((char*)&mdr, sizeof(struct MsgDetachReply), event->getUID());
	}
	else{
		for(int i=0; i<(int)event->getNGUID(); i++){
			//mSM->addGUID(event->getGUID(i));
			mSockTable.removeGUID(event->getUID(), event->getGUID(i));
		}
		mdr.UID = event->getUID();
		mdr.reqID = event->getReqID();
		mdr.retValue = 0;
		ms.send((char*)&mdr, sizeof(struct MsgDetachReply), event->getUID());
	}
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleDetach end of event");
}

/*
 * Close the socket
 */
void MF_Main::handleClose(MF_APIClose *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleClose");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleClose start of event");
	struct MsgCloseReply cloR;
	MF_SocketManager *mSM = mSockTable.getById(event->getUID());
	cloR.type = MSG_TYPE_CLOSE_REPLY;
	cloR.UID = event->getUID();
	cloR.reqID = event->getReqID();
	if(!mSM){
		cloR.retValue = UID_NEXISTS;
	}
	else{
		//I think the manager should not be deleted but only removed
		mSockTable.removeManager(event->getUID());
		mSM->closeRequest();
		cloR.retValue = 0;
		delete mSM;
	}
	ms.send((char *)&cloR, sizeof(struct MsgCloseReply), event->getUID());
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleClose end of event");
}



/********************************
 Operations to handle the events from lower layers (network interfaces)
********************************/


void MF_Main::handleIfStateUpdate(MF_EvIfStateUpdate *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleIfStateUpdate");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleIfStateUpdate start of event");
	mRouter.sendAssocReqs();
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleIfStateUpdate end of event");
}

/*
 * Handles a csyn
 */
void MF_Main::handleCsyn(MF_EvCysn *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleCsyn");
	MF_DeviceManager *dm = mNetManager.getByIndex(event->getInterface());
	if(dm!=NULL && dm->isActive()){
		dm->cysnMain(event->getHopID(), event->getPktCount());
	}
	else{
		MF_Log::mf_log(MF_INFO, "MF_Main::handleCsyn the interface specified is not available");
	}
}

/*
 * Handles a csyn ack
 */
void MF_Main::handleCsynAck(MF_EvCysnAck *event) {
	MF_Log::mf_log(MF_TIME, "MF_Main:handleCsynAck SEND_CSYN END ack received for interface %d", event->getInterface());
	MF_DeviceManager *dm = mNetManager.getByIndex(event->getInterface());
	if(dm!=NULL && dm->isActive()){
		dm->cysnReceived(event->getHopID(), event->getPktCount(), event->getBitmap());
	}
	else{
		MF_Log::mf_log(MF_INFO, "MF_Main:handleCsynAck the interface specified is not available");
	}
}

/*
 * Handles a message generated when a csyn message has timed out
 */
void MF_Main::handleCsynTimeout(MF_EvCysnTOut *event) {
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleCsynTimeout");
	MF_DeviceManager *dm = mNetManager.getByIndex(event->getInterface());
	//TODO Manage possible switch of interface
	if(dm!=NULL){
		dm->cysnTimeOut(event->getChunkID());
	}
}


/*
 * Handles a new Link Probe from an interface
 */
void MF_Main::handleLinkProbe(MF_EvLinkProb *event){
	MF_Log::mf_log(MF_DEBUG, "MF_Main:handleLinkProbe");
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleLinkProbe start of event");
	MF_DeviceManager *dm = mNetManager.getByIndex(event->getDevID());
	if(dm==NULL){
		MF_Log::mf_log(MF_INFO, "MF_Main:handleLinkProbe no device with if number %d", event->getDevID());
		delete event;
		return;
	}
	string mac((const char*)event->getEth());
	struct in_addr addr;
	addr.s_addr = event->getIP();
	string ip(inet_ntoa(addr));
	if(dm->updateProb(mac, ip, event->getTimestamp())){
		dm->sendLinkProb(mRouter.getDefaultGUID(), event->getSLP(), event->getSeqNum());
	}
	//MF_Log::mf_log(MF_TIME, "MF_Main:handleLinkProbe end of event");
}

void MF_Main::handleChkSent(MF_EvChkComplete *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleChkSent TRANSMIT_TIME END chunk %lu interface %d", event->getChunkInfo()->getChunkID(), event->getInterface());
	MF_SocketManager *sm = mSockTable.getByTID(event->getChunkInfo()->getOwner());
	if(sm==NULL){
		MF_Log::mf_log(MF_DEBUG, "MF_Main:handleChkSent chunk for transport that doesn't exist\n");
		//TODO free the memory of the chunk or will be lost unused
		return;
	}
	sm->releaseChunk(event->getChunkInfo());
}


/*
 * Handles a message that says that a new chunk has been received
 */
void MF_Main::handleUpIf(MF_EvUpIf *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpIf TRAVERSE_UP_TIME INIT");
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpIf ROUTER_UP_TIME INIT");
	bool ret = mRouter.procRecv(event->getChunkInfo());
	if(ret){
		MF_DeviceManager *mdm = mNetManager.getByIndex((int)event->getChunkInfo()->getOwner());
		if(mdm!=NULL){
			mdm->releaseChunk(event->getChunkInfo()->getHopID());
		}
	}
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpIf ROUTER_UP_TIME END");
}


/*
 * From Router to Transport
 */
void MF_Main::handleUpRout(MF_EvUpRout *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpRout TRANSPORT_UP_TIME INIT");
	MF_Transport *t = event->getTransport();
	if(t == NULL){
		MF_Log::mf_log(MF_WARN, "MF_Main:handleUpRout the transport does not exist");
		//MF_Log::mf_log(MF_TIME, "MF_Main:handleUpRout end of event");
		return;
	}
	t->recvData(event->getChunkInfo());
	MF_DeviceManager *mdm = mNetManager.getByIndex((int)event->getChunkInfo()->getOwner());
	if(mdm!=NULL){
		mdm->releaseChunk(event->getChunkInfo()->getHopID());
	}
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpRout TRANSPORT_UP_TIME END");
}


/*
 * From Transport to API
 */
void MF_Main::handleUpTransp(MF_EvUpTransp *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpTransp RECV_TIME INIT");
	MF_SocketManager *sm = mSockTable.getByTID(event->getTID());
	if(!sm){
		MF_Log::mf_log(MF_DEBUG, "MF_Main: chunk for transport that doesn't exist\n");
		//MF_Log::mf_log(MF_TIME, "MF_Main:handleUpTransp end of event");
		return;
	}
	//add chunk to available ones for the socket manager
	//other option is to keep them in the transport and get them once they can be transferd up to the api
	sm->msgAvailable(event);
	MF_Log::mf_log(MF_TIME, "MF_Main:handleUpTransp RECV_TIME END");
}


/*
 * From API to Transport
 */
void MF_Main::handleDownAPI(MF_EvDownAPI *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownAPI TRAVERSE_DOWN_TIME INIT");
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownAPI TRANSPORT_DOWN_TIME INIT");
	MF_Transport *t = event->getTransport();
	t->sendData(event->getData(), event->getSize(), event->getSrcGUID(), event->getDstGUID(), event->getOpts());
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownAPI TRANSPORT_DOWN_TIME END");
}


/*
 * From Transport to Router
 */
void MF_Main::handleDownTransp(MF_EvDownTransp *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownTransp ROUTER_DOWN_TIME INIT");
	mRouter.procSend(event->getChunkInfo());
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownTransp ROUTER_DOWN_COMPLETE END");
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownTransp TRAVERSE_DOWN_TIME END");
}


/*
 * From Router to Interface
 */
void MF_Main::handleDownRout(MF_EvDownRout *event){
	MF_Log::mf_log(MF_TIME, "MF_Main:handleDownRout TRANSMIT_TIME INIT chunk %lu interface %d", event->getChunkInfo()->getChunkID(), event->getDevID());
	MF_DeviceManager *dm = mNetManager.getByIndex(event->getDevID());
	if(dm){
		dm->sendChunk(event->getChunkInfo());
	}
}
